/*
 * @(#)AbstractProcessor.java
 *
 * Copyright (c) 2001-2002, JangHo Hwang
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 	1. Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 *
 * 	2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 *
 * 	3. Neither the name of the JangHo Hwang nor the names of its contributors
 * may be used to endorse or promote products derived from this software
 * without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *    $Id: AbstractProcessor.java,v 1.6 2002/03/16 17:44:30 xrath Exp $
 */
package rath.msnm;

import java.lang.reflect.Method;
import java.io.*;
import java.net.Socket;
import java.util.HashMap;
import java.util.Iterator;

import rath.msnm.entity.Callback;
import rath.msnm.entity.ServerInfo;
import rath.msnm.msg.IncomingMessage;
import rath.msnm.msg.OutgoingMessage;
import rath.msnm.msg.MimeMessage;
/**
 *  DS, NS, SS  Ǵ κ ߻
 *  Ŭ̴.
 * <p>
 *  ش Processor ױ  OUT .
 *  ʰ Ϸ setAutoOutSend(false)  ϶.
 *
 * @author Jang-Ho Hwang, rath@linuxkorea.co.kr
 * @version $Id: AbstractProcessor.java,v 1.6 2002/03/16 17:44:30 xrath Exp $
 */
public abstract class AbstractProcessor extends Thread
{
	public final MSNMessenger msn;
	private static final Class[] PARAM_TYPE = new Class[] { IncomingMessage.class };

	private ServerInfo info = null;
	private boolean autoOut = false;
	private int trId;
	private String name = "";

	protected HashMap callbackMap = new HashMap();
	protected Socket socket = null;
	protected boolean isLive;
	protected BufferedReader in = null;
	protected PrintWriter out = null;

	/**
	 * Client processor ƴ Server processor Ҷ , ServerInfo ʿ
	 *  ڸ Ͽ νϽ ϵ Ѵ.
	 */
	protected AbstractProcessor( MSNMessenger msn )
	{
		this( msn, null, 1 );
	}

	/**
	 * ־ ServerInfo  Processor Ѵ.
	 * ⺻ Ʈ ̵ 0 ۵ȴ.
	 */
	protected AbstractProcessor( MSNMessenger msn, ServerInfo info )
	{
		this( msn, info, 1 );
	}

	/**
	 * ־ ServerInfo  trId  Ʈ ̵
	 *  Processor Ѵ.
	 */
	protected AbstractProcessor( MSNMessenger msn, ServerInfo info, int trId )
	{
		this.msn = msn;
		this.info = info;
		this.trId = trId;
	}

	public void setServerName( String name )
	{
		this.name = name;
		setName( "MSN Channel(" + name + ")" );
	}

	public String getServerName()
	{
		return this.name;
	}

	/**
	 * ش processor ɶ OUT ޽ ڵ 
	 * Ѵ. ⺻ false̴.
	 */
	public void setAutoOutSend( boolean autoOut )
	{
		this.autoOut = autoOut;
	}

	/**
	 * ش processor ɶ OUT ޽ ڵϰ Ǿִ
	 * ȮѴ.
	 */
	public boolean isAutoOutSend()
	{
		return this.autoOut;
	}

	protected void setServerInfo( ServerInfo info )
	{
		this.info = info;
	}

	protected ServerInfo getServerInfo()
	{
		return this.info;
	}

	/**
	 * ش server tcp/ip  ΰ,  Ʈ Ѵ.
	 *  UTF-8 ڵ ⺻ ϵ Ǿִ.
	 */
	protected void makeConnection() throws IOException
	{
		this.socket = new Socket( info.getHostAddress(), info.getPort() );
		this.in = new BufferedReader( new InputStreamReader(
			socket.getInputStream(), "UTF-8") );
		this.out = new PrintWriter( new OutputStreamWriter(
			socket.getOutputStream(), "UTF-8") );
	}

	/**
	 *   ξ  ־   ִٸ
	 * ̰ ֵ Ѵ.
	 */
	public abstract void init() throws IOException;

	/**
	 *  ϴ ޽ ϰ, trId ϳ Ų.
	 */
	public synchronized void sendMessage( OutgoingMessage msg ) throws IOException
	{
		if( out==null )
			return;

		if( trId!=-1 && msg.getBackProcess()!=null )
		{
			callbackMap.put( new Integer(trId), msg.getBackProcess() );
		}

		if( Debug.printOutput )
			System.out.println( "=> " + msg.toString() );

		out.print( msg.toString() );
		out.print( "\r\n" );
		out.flush();
		trId++;
	}

	/**
	 * MIME޽   Ѵ.
	 */
	public synchronized void sendMimeMessage( OutgoingMessage msg, MimeMessage mime ) throws IOException
	{
		if( trId!=-1 && msg.getBackProcess()!=null )
		{
			callbackMap.put( new Integer(trId), msg.getBackProcess() );
		}

		byte[] raw = mime.getBytes();
		int len = raw.length;

		msg.add( len );
		out.print( msg.toString() );
		out.print( "\r\n" );
		String s = new String(raw,"UTF-8");
		out.print( s );
		if( Debug.printMime )
			System.out.println( s );
		out.flush();
		trId++;
	}

	public synchronized void sendCHLResponse( OutgoingMessage msg, String hash ) throws IOException
	{
		out.print( msg.toString() );
		out.print( "\r\n" );
		out.print( hash );
		out.flush();
		trId++;
	}

	/**
	 * ش ޽  trId ش.
	 *  trId  1 ϰ ȴ.
	 */
	public void markTransactionId( OutgoingMessage msg )
	{
		msg.setTransactionId( this.trId );
	}

	/**
	 *  ۵Ǿ trId ȯѴ.
	 */
	public int getCurrentTransactionId()
	{
		return this.trId;
	}

	protected Method lookupMethod( Callback cb ) throws NoSuchMethodException,
		SecurityException
	{
		return cb.getClassRef().getMethod( cb.getMethodName(), PARAM_TYPE );
	}

	/**
	 *  callback ϽŲ.
	 */
	protected void registerCallback( Integer trId, Callback cb )
	{
		callbackMap.put( trId, cb );
	}

	/**
	 * κ ϴ ޽ (Ϲ push ƴ) óϴ κ̴.
	 */
	public void processMessage( IncomingMessage msg ) throws Exception
	{

	}

	/**
	 * Serverκ Ϲ notifyǴ ޽ óϴ κ̴.
	 *  ޼ҵ带  ȣ  ϶.
	 */
	public void processNotifyMessage( IncomingMessage msg ) throws Exception
	{
		if( msg.getHeader().equals("MSG") )
		{
			/*
			 * MSG UserHandle FriendlyName Length\r\nMessage
			 */
			MimeMessage mime = new MimeMessage(/*msg.get(0), msg.get(1)*/);
			int len = msg.getInt(2);
			int readlen = 0;
			byte[] b = new byte[ len ];
			while( readlen < len )
			{
				String line = in.readLine();
				byte[] lb = line.getBytes("UTF-8");
				System.arraycopy( lb, 0, b, readlen, lb.length );
				b[readlen+lb.length] = '\r';
				b[readlen+lb.length+1] = '\n';
				readlen += lb.length + 2/*CRLF*/;
			}
			String s = new String(b, 0, readlen, "UTF-8");
			mime.parse( s );
			if( Debug.printMime )
				System.out.println( s );
		}
	}

	/**
	 *  Callback ̿ trId   Ǹ
	 *  ޼ҵ带  ̻ callback map  ʵ
	 * Ͽ Ѵ.
	 */
	public void removeInfiniteTransactionId( int trId )
	{
		callbackMap.remove( new Integer(trId) );
	}

	/**
	 * Thread loop κ
	 */
	public final void run()
	{
		this.isLive = true;
		try
		{
			makeConnection();
			init();

			while( isLive )
			{
				String line = in.readLine();
				if( Debug.printInput )
					System.out.println( "<= " + line );
				if( line==null )
					break;

				IncomingMessage msg = null;
				try
				{
					msg = IncomingMessage.getInstance(line);
				}
				catch( Exception e ) {}
				if( msg==null ) continue;

				if( !msg.isNotify() )
				{
					Integer trId = new Integer(msg.getTransactionId());
					Callback cb = (Callback)callbackMap.get( trId );
					if( cb!=null )
					{
						if( !cb.isInfinite() )
							callbackMap.remove( trId );

						lookupMethod(cb).invoke( this, new Object[]{msg} );
					}
					else
						processMessage( msg );
				}
				else
					processNotifyMessage( msg );

			}
		}
		catch( Throwable e )
		{
			processError( e );
		}
		finally
		{
			cleanUp();

			if( in!=null )
			{
				try{ in.close(); }
				catch( Exception e ) {}
				in = null;
			}
			if( out!=null )
			{
				try
				{
					if( autoOut )
						sendMessage(new OutgoingMessage("OUT"));
					out.close();
				}
				catch( Exception e ) {}
				out = null;
			}
			if( socket!=null )
			{
				try{ socket.close(); }
				catch( Exception e ) {}
				socket = null;
			}
		}
	}

	/**
	 * 尡 ǰ Ʈ ݱ  ȣȴ.
	 */
	public abstract void cleanUp();

	/**
	 * ش Processor ߻Ǵ  ܴ ̰ óѴ.
	 *  ٸ ó ϰ ʹٸ, overriding϶.
	 */
	public void processError( Throwable e )
	{
		System.err.println( this.getClass().getName() + ":" );
		e.printStackTrace();
	}
};
