/*
 * @(#)MSNMessenger.java
 *
 * Copyright (c) 2001-2002, JangHo Hwang
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 	1. Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 *
 * 	2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 *
 * 	3. Neither the name of the JangHo Hwang nor the names of its contributors
 * may be used to endorse or promote products derived from this software
 * without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *    $Id: MSNMessenger.java,v 1.11 2002/03/16 17:45:30 xrath Exp $
 */
package rath.msnm;

import java.io.*;
import java.util.ArrayList;
import java.util.Enumeration;
import java.util.Iterator;
import java.util.Vector;
import java.util.Hashtable;
import java.util.Properties;

import rath.msnm.entity.ServerInfo;
import rath.msnm.entity.MsnFriend;
import rath.msnm.event.MsnAdapter;
import rath.msnm.event.MsnListener;
import rath.msnm.msg.MimeMessage;
import rath.msnm.msg.FileTransferMessage;
import rath.msnm.ftp.ToSendFile;
import rath.msnm.ftp.VolatileTransferServer;
import rath.msnm.ftp.VolatileDownloader;
/**
 * MSN ޽  ϰ ̰ ϱ 
 * Entry point Ǵ Ŭ̴.
 * login ûϰ loginComplete ̺Ʈ ߻ ĺ
 * ̰ ޼ҵ带   ִ.
 * ׷  NS proc ̻ NullPointerException ̴.
 * <p>
 * <pre><code>
 *  MSNMessenger msn = new MSNMessenger( "xiguel@hotmail.com", "12341234" );
 *  msn.setInitialStatus( UserStatus.ONLINE );
 *  msn.addMsnListener( new MsnAdapter() {
 *      public void progressTyping( SwitchboardSession ss,
 *          MsnFriend friend, String typingUser )
 *      {
 *          System.out.println( "Typing on " + friend.getLoginName() );
 *      }
 *      public void instantMessageReceived( SwitchboardSession ss,
 *          MsnFriend friend, MimeMessage mime )
 *      {
 *          System.out.println( "*** MimeMessage from " + friend.getLoginName() );
 *          System.out.println( mime.getMessage() );
 *          System.out.println( "*****************************" );
 *      }
 *  });
 *  msn.login();
 * </code></pre>
 *
 * @author Jang-Ho Hwang, rath@linuxkorea.co.kr
 * @version $Id: MSNMessenger.java,v 1.11 2002/03/16 17:45:30 xrath Exp $
 */
public class MSNMessenger
{
	private DispatchProcessor ds = null;
	private NotificationProcessor ns = null;
	private BuddyGroup buddyGroup = null;
	private BuddyList forwardList = null;
	private LocalCopy localCopy = null;

	boolean isLogged = false;

	private String loginName = null;
	private String password = null;
	private String initStatus = UserStatus.ONLINE;
	private MsnListener base = null;
	protected ArrayList listeners = new ArrayList();

	private MsnFriend owner = null;
	private Hashtable sessionMap = new Hashtable();

	/**
	 * MSNMessenger ü Ѵ.
	 */
	public MSNMessenger()
	{
		this( null, null );
	}

	/**
	 * ־ account MSNMessenger ü Ѵ.
	 *
	 * @param  loginName   login ̸. (e.g. windrath@hotmail.com)
	 * @param  password   ڽ password
	 */
	public MSNMessenger( String loginName, String password )
	{
		this.loginName = loginName;
		this.password = password;

		this.base = new Listener();
		this.buddyGroup = BuddyGroup.getInstance();
		this.forwardList = buddyGroup.getForwardList();
		this.localCopy = new LocalCopy();
	}

	private void initLogon()
	{
		buddyGroup.clear();
		localCopy.setLoginName( loginName );

		/* ֱ øѹ NS Ѵ */
		localCopy.loadInformation();
		ns.lastSerial = Integer.parseInt( localCopy.getProperty("SerialNumber","0") );

		/* ֱ  Ʈ ´. */
		localCopy.loadBuddies( buddyGroup );
	}

	/**
	 * ο ø ȣ  , BuddyList Ǿٴ ̺Ʈ ߼Ѵ.
	 *
	 * @param serial
	 * @param fireEvent
	 */
	void storeLocalCopy( int serial )
	{
		localCopy.setProperty( "SerialNumber", String.valueOf(serial) );
		localCopy.storeInformation();
		localCopy.storeBuddies( buddyGroup );

		fireBuddyListModifiedEvent();
	}

	public BuddyGroup getBuddyGroup()
	{
		return this.buddyGroup;
	}

	public LocalCopy getLocalCopy()
	{
		return this.localCopy;
	}

	/**
	 * α Ҷ ʱ ° Ѵ. ⺻δ '¶'̴.
	 * ̰   ִ ° {@link UserStatus UserStatus} ̽
	 * Ǿִ  ϸ ȴ.
	 */
	public void setInitialStatus( String code )
	{
		this.initStatus = code;
	}

	/**
	 *   ʱ ڵ带 ´.
	 */
	public String getInitialStatus()
	{
		return this.initStatus;
	}

	/**
	 *  α   ȮѴ.
	 */
	public boolean isLoggedIn()
	{
		return this.isLogged;
	}

	/**
	 * κ ߻Ǵ ̺Ʈ ޽ ó MsnListner
	 * ̽ Ѵ.  multi-listener ؾ,
	 *   Listener  Ѵ. ׷Ƿ
	 * ݵ ʸ ؾѴ.
	 */
	public synchronized void addMsnListener( MsnListener l )
	{
		if( !listeners.contains(l) )
			listeners.add(l);
	}

	public void fireInstantMessageEvent( SwitchboardSession ss, MsnFriend friend,
		MimeMessage mime )
	{
		base.instantMessageReceived(ss, friend, mime);
		for(int i=listeners.size()-1; i>=0; i--)
			((MsnListener)listeners.get(i)).instantMessageReceived(ss, friend, mime);
	}

	public void fireJoinSessionEvent( SwitchboardSession ss, MsnFriend friend )
	{
		base.whoJoinSession(ss, friend);
		for(int i=listeners.size()-1; i>=0; i--)
			((MsnListener)listeners.get(i)).whoJoinSession(ss, friend);
	}

	public void fireListOnlineEvent( MsnFriend friend )
	{
		base.listOnline(friend);
		for(int i=listeners.size()-1; i>=0; i--)
			((MsnListener)listeners.get(i)).listOnline(friend);
	}

	public void fireLoginCompleteEvent( MsnFriend own )
	{
		base.loginComplete(own);
		for(int i=listeners.size()-1; i>=0; i--)
			((MsnListener)listeners.get(i)).loginComplete(own);
	}

	public void firePartSessionEvent( SwitchboardSession ss, MsnFriend friend )
	{
		base.whoPartSession(ss, friend);
		for(int i=listeners.size()-1; i>=0; i--)
			((MsnListener)listeners.get(i)).whoPartSession(ss, friend);
	}

	public void fireProgressTypingEvent( SwitchboardSession ss, MsnFriend friend,
		String typeuser )
	{
		base.progressTyping(ss, friend, typeuser);
		for(int i=listeners.size()-1; i>=0; i--)
			((MsnListener)listeners.get(i)).progressTyping(ss, friend, typeuser);
	}

	public void fireSwitchboardSessionStartedEvent( SwitchboardSession ss )
	{
		base.switchboardSessionStarted(ss);
		for(int i=listeners.size()-1; i>=0; i--)
			((MsnListener)listeners.get(i)).switchboardSessionStarted(ss);
	}

	public void fireSwitchboardSessionEndedEvent( SwitchboardSession ss )
	{
		base.switchboardSessionEnded(ss);
		for(int i=listeners.size()-1; i>=0; i--)
			((MsnListener)listeners.get(i)).switchboardSessionEnded(ss);
	}

	public void fireSwitchboardSessionAbandonEvent( SwitchboardSession ss, String targetName )
	{
	    base.switchboardSessionAbandon(ss, targetName);
		for(int i=listeners.size()-1; i>=0; i--)
			((MsnListener)listeners.get(i)).switchboardSessionAbandon(ss, targetName);
	}

	public void fireUserOnlineEvent( MsnFriend friend )
	{
		base.userOnline(friend);
		for(int i=listeners.size()-1; i>=0; i--)
			((MsnListener)listeners.get(i)).userOnline(friend);
	}

	public void fireUserOfflineEvent( String loginName )
	{
		base.userOffline(loginName);
		for(int i=listeners.size()-1; i>=0; i--)
			((MsnListener)listeners.get(i)).userOffline(loginName);
	}

	public void fireFilePostedEvent( SwitchboardSession ss, int cookie, String filename,
		int filesize )
	{
		base.filePosted(ss, cookie, filename, filesize);
		for(int i=listeners.size()-1; i>=0; i--)
			((MsnListener)listeners.get(i)).filePosted(ss, cookie, filename, filesize);
	}

	public void fireFileSendAcceptedEvent( SwitchboardSession ss, int cookie )
	{
		base.fileSendAccepted(ss, cookie);
		for(int i=listeners.size()-1; i>=0; i--)
			((MsnListener)listeners.get(i)).fileSendAccepted(ss, cookie);
	}

	public void fireFileSendRejectedEvent( SwitchboardSession ss, int cookie, String reason )
	{
		base.fileSendRejected(ss, cookie, reason);
		for(int i=listeners.size()-1; i>=0; i--)
			((MsnListener)listeners.get(i)).fileSendRejected(ss, cookie, reason);
	}

	public void fireFileSendStartedEvent( VolatileTransferServer server )
	{
		base.fileSendStarted(server);
		for(int i=listeners.size()-1; i>=0; i--)
			((MsnListener)listeners.get(i)).fileSendStarted(server);
	}

	public void fireFileReceiveStartedEvent( VolatileDownloader down )
	{
		base.fileReceiveStarted(down);
		for(int i=listeners.size()-1; i>=0; i--)
			((MsnListener)listeners.get(i)).fileReceiveStarted(down);
	}

	public void fireFileSendErrorEvent( VolatileTransferServer server, Throwable e )
	{
		base.fileSendError(server, e);
		for(int i=listeners.size()-1; i>=0; i--)
			((MsnListener)listeners.get(i)).fileSendError(server, e);
	}

	public void fireFileReceiveErrorEvent( VolatileDownloader down, Throwable e )
	{
		base.fileReceiveError(down, e);
		for(int i=listeners.size()-1; i>=0; i--)
			((MsnListener)listeners.get(i)).fileReceiveError(down, e);
	}

	public void fireWhoAddedMeEvent( MsnFriend friend )
	{
		base.whoAddedMe( friend );
		for(int i=listeners.size()-1; i>=0; i--)
			((MsnListener)listeners.get(i)).whoAddedMe( friend );
	}

	public void fireWhoRemovedMeEvent( MsnFriend friend )
	{
		base.whoRemovedMe( friend );
		for(int i=listeners.size()-1; i>=0; i--)
			((MsnListener)listeners.get(i)).whoRemovedMe( friend );
	}

	public void fireBuddyListModifiedEvent()
	{
		base.buddyListModified();
		for(int i=listeners.size()-1; i>=0; i--)
			((MsnListener)listeners.get(i)).buddyListModified();
	}

	public void fireAddFailedEvent( int errcode )
	{
		base.addFailed( errcode );
		for(int i=listeners.size()-1; i>=0; i--)
			((MsnListener)listeners.get(i)).addFailed( errcode );
	}

	public void fireLoginErrorEvent( String header )
	{
		base.loginError(header);
		for(int i=listeners.size()-1; i>=0; i--)
			((MsnListener)listeners.get(i)).loginError(header);
	}

	public void fireRenameNotifyEvent( MsnFriend friend )
	{
		base.renameNotify( friend );
		for(int i=listeners.size()-1; i>=0; i--)
			((MsnListener)listeners.get(i)).renameNotify(friend);
	}

	public void fireAllListUpdatedEvent()
	{
		base.allListUpdated();
		for(int i=listeners.size()-1; i>=0; i--)
			((MsnListener)listeners.get(i)).allListUpdated();
	}

	public void fireLogoutNotifyEvent()
	{
	    base.logoutNotify();
		for(int i=listeners.size()-1; i>=0; i--)
			((MsnListener)listeners.get(i)).logoutNotify();
	}

	/**
	 * ش ̺Ʈʸ Ѵ.
	 */
	public synchronized void removeMsnListener( MsnListener l )
	{
		listeners.remove(l);
	}

	/**
	 *  α ̸(LoginName) ȯѴ.
	 */
	public String getLoginName()
	{
		return this.loginName;
	}

	/**
	 *  йȣ ȯѴ.
	 */
	public String getPassword()
	{
		return this.password;
	}

	/**
	 * ڱ ڽ MsnFriend νϽ ȯѴ.
	 *  α ʾҴٸ, null ȯ ̴.
	 */
	public MsnFriend getOwner()
	{
		return this.owner;
	}

	/**
	 * DS  Ѵ.
	 * @deprecated ο MSN DS  Ƿ  Ѵ.
	 */
	private void dispatch()
	{
		ds = new DispatchProcessor( this,
			ServerInfo.getDefaultDispatchServerInfo() );
		ds.start();
	}

	/*
	 *  NS  Ѵ.
	 */
	protected void loginImpl()
	{
		this.ns = new NotificationProcessor( this, ServerInfo.getDefaultServerInfo(), 1 );
		initLogon();
		this.ns.start();
	}

	/**
	 * ־ ̸ йȣ α Ѵ.
	 * ̷  ڿ  ̸ йȣ õȴ.
	 */
	public void login( String username, String password )
	{
		this.loginName = username;
		this.password = password;

		loginImpl();
	}

	/**
	 * α Ѵ.
	 */
	public void login()
	{
		if( loginName==null || password==null )
			throw new IllegalArgumentException( "Login name and password must not be null" );
		login( this.loginName, this.password );
	}

	/**
	 * ִ  Switchboard session  ϰ
	 * DS, NS Logout   Ѵ.
	 */
	public void logout()
	{
		Enumeration e = sessionMap.elements();
		while( e.hasMoreElements() )
		{
			SwitchboardSession ss = (SwitchboardSession)e.nextElement();
			ss.interrupt();
			ss.cleanUp();
		}
		sessionMap.clear();

		ds = null;
		if( ns!=null )
		{
			ns.interrupt();
			try	{
				ns.logout();
			} catch( IOException ex ) {}
			ns = null;
		}
	}

	/**
	 * ڽ ¸ Ѵ.  ڿ UserStatus ̽
	 * ǵǾִ ڿ Ͽ߸ Ѵ.
	 * α  Ŀ Default ¶ ° Ǿ ̴.
	 */
	public void setMyStatus( String status ) throws IOException
	{
		this.ns.setMyStatus( status );
	}

	/**
	 *  ڱ ڽ ڵ尪 ´.
	 */
	public String getMyStatus()
	{
		return this.ns.getMyStatus();
	}

	public void setMyFriendlyName( String newName ) throws IOException
	{
		this.ns.setMyFriendlyName( newName );
	}

	/**
	 * ȭû Ǵ. 񵿱 óǱ ,
	 *  ޼ҵ ۾ ٰ ؼ  ̷  ƴϴ.
	 *  ޼ҵ ٷ return ȴ.
	 * <p>
	 *   ̷µ 2-3  ð ɸ.
	 */
	public void doCall( String loginName ) throws IOException
	{
		ns.doCallFriend( loginName );
	}

	public void addFriend( String loginName ) throws IOException
	{
		ns.requestAdd( loginName );
	}

	public void removeFriend( String loginName ) throws IOException
	{
		ns.requestRemove( loginName );
	}

	public void addGroup( String groupName ) throws IOException
	{
		ns.requestCreateGroup( groupName );
	}

	public void removeGroup( int groupIndex ) throws IOException
	{
	    ns.requestRemoveGroup( groupIndex );
	}

	public void renameGroup( int groupIndex, String newName ) throws IOException
	{
	    ns.requestRenameGroup( groupIndex, newName );
	}

	/**
	 * ־ ģ ׷ε old new ̵Ѵ.
	 */
	public void moveGroupAsFriend( MsnFriend friend, int oldIndex, int newIndex )
		throws IOException
	{
		ns.requestMoveGroup( friend, oldIndex, newIndex );
	}

	/**
	 * doCall   ,  ɶ  ٸٴ 
	 * ٸ.
	 * <p>
	 * Object.wait ޼ҵ带 Ͽ ٸ ǰ,   ޽ ö
	 *  ϰ ȴ.
	 */
	public SwitchboardSession doCallWait( String loginName )
		throws IOException, InterruptedException
	{
		return ns.doCallFriendWait( loginName );
	}

	/**
	 * ش loginName Ե   ù°  ã ȯѴ.
	 *  ׷   ʴ´ٸ, null ȯѴ.
	 *
	 * @return  loginName Ե  ٸ null ȯ.
	 */
	public SwitchboardSession findSwitchboardSession( String loginName )
	{
		Enumeration e = sessionMap.elements();
		while( e.hasMoreElements() )
		{
			SwitchboardSession ss = (SwitchboardSession)e.nextElement();
			if( ss.isInFriend(loginName) )
				return ss;
		}
		return null;
	}

	/**
	 * ش loginName 1:1   ãش.
	 *    ʴ´ٸ, null ȯѴ.
	 */
	public SwitchboardSession findSwitchboardSessionAt( String loginName )
	{
		Enumeration e = sessionMap.elements();
		while( e.hasMoreElements() )
		{
			SwitchboardSession ss = (SwitchboardSession)e.nextElement();
			if( ss.getFriendCount()==1 && ss.isInFriend(loginName) )
				return ss;
		}
		return null;
	}

	/**
	 * ־ loginName  ڿ MIME ޽ Ѵ.
	 *  loginName  ڿ  session  ʴ´ٸ,
	 *  false ȯ ̴.
	 *
	 * @return true -  Ͽ,
	 *         false - Ⱑ .
	 */
	public boolean sendMessage( String loginName, MimeMessage msg ) throws IOException
	{
		SwitchboardSession ss = findSwitchboardSession(loginName);
		if( ss!=null )
		{
			ss.sendMessage( msg );
			return true;
		}
		return false;
	}

	/**
	 * ־ loginName  ڿ MIME ޽ Ѵ.
	 *  loginName   sessionId ġϴ θ Ѵ.
	 *  ġϴ  ٸ ޽ ޵  ̸,
	 * sessionId null̶ ù° ߰ߵǴ  ǿ ޵ ̴.
	 * <p>
	 *  sessionId null̰ loginName ϴ   ʴ´ٸ
	 * ۵ ʰ false ȯѴ.
	 *
	 * @return true -  Ͽ,
	 *         false - Ⱑ .
	 */
	public boolean sendMessage( String loginName, MimeMessage msg, String sessionId )
		throws IOException
	{
		SwitchboardSession ss = (SwitchboardSession)sessionMap.get(sessionId);
		if( ss==null )
		{
			return sendMessage(loginName, msg);
		}
		ss.sendMessage( msg );
		return true;
	}

	/**
	 * ־ loginName  ڿ MIME ޽ Ѵ.
	 *  ġϴ  ٸ ޽ ޵  ̸,
	 * session null̶ ۵  ̴.
	 *
	 * @return true -  Ͽ,
	 *         false - Ⱑ .
	 */
	public boolean sendMessage( MimeMessage msg, SwitchboardSession session )
		throws IOException
	{
		if( session!=null )
		{
			session.sendMessage( msg );
			return true;
		}
		return false;
	}

	/**
	 * ش sessionId   ϱ  loginName
	 *  ۽ſû ޽ .
	 *
	 */
	public void sendFileRequest( String loginName, File file, String sessionId )
		throws IOException
	{
		if( sessionId==null )
			throw new IllegalArgumentException( "session id must not be null" );

		sendFileRequest( loginName, file, (SwitchboardSession)sessionMap.get(sessionId) );
	}

	/**
	 * ش sessionId   ϱ  loginName
	 *  ۽ſû ޽ .
	 *
	 */
	public void sendFileRequest( String loginName, File file, SwitchboardSession session )
		throws IOException
	{
		if( session==null )
			throw new IllegalArgumentException( "session must not be null" );

		FileTransferMessage msg = FileTransferMessage.createInviteMessage(file);
		ToSendFile toSend = new ToSendFile( msg.getProperty("Invitation-Cookie"), loginName, file );
		session.sendFileRequest( toSend, msg );
	}

	/////////////////////////////////////////////////////////////////////

	private class Listener extends MsnAdapter
	{
		public void renameNotify( MsnFriend own )
		{
			if( own!=null && owner.getLoginName().equals(own.getLoginName()) )
				owner.setFriendlyName( own.getFriendlyName() );
		}
		public void loginComplete( MsnFriend own )
		{
			isLogged = true;
			owner = own;
		}

		public void listOnline( MsnFriend friend )
		{
			/*
			 * add ޼ҵ, ̹ ϴ ڶ  ϵ Ǿִ.
			 */
			forwardList.add( friend );
		}

		public void userOnline( MsnFriend friend )
		{
			forwardList.add( friend );
		}

		public void userOffline( String loginName )
		{
			forwardList.setOffline( loginName );
		}

		public void switchboardSessionStarted( SwitchboardSession ss )
		{
			String sid = ss.getSessionId();
			if( sid==null )
				return;

			sessionMap.put( sid, ss );
			/*  ۵Ǿٰ  ݵ °  ƴϴ. */
		}

		public void switchboardSessionEnded( SwitchboardSession ss )
		{
			String sid = ss.getSessionId();
			sessionMap.remove( sid );
		}

		public void whoAddedMe( MsnFriend friend )
		{
			System.out.println( friend + " Add me." );
		}

		public void whoRemovedMe( MsnFriend friend )
		{
			System.out.println( friend.getLoginName() + " remove me." );
		}
	}
}
