/*
 * @(#)SwitchboardSession.java
 *
 * Copyright (c) 2001-2002, JangHo Hwang
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 	1. Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 *
 * 	2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 *
 * 	3. Neither the name of the JangHo Hwang nor the names of its contributors
 * may be used to endorse or promote products derived from this software
 * without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *    $Id: SwitchboardSession.java,v 1.8 2002/03/16 17:44:30 xrath Exp $
 */
package rath.msnm;

import java.lang.reflect.Method;
import java.io.CharArrayWriter;
import java.io.IOException;
import java.util.Collection;
import java.util.HashMap;

import rath.msnm.entity.Callback;
import rath.msnm.entity.MsnFriend;
import rath.msnm.entity.ServerInfo;
import rath.msnm.msg.IncomingMessage;
import rath.msnm.msg.OutgoingMessage;
import rath.msnm.msg.MimeMessage;
import rath.msnm.msg.FileTransferMessage;
import rath.msnm.ftp.ToSendFile;
import rath.msnm.ftp.FileMessageProcessor;
/**
 * ģ ȭ Ǵ νƮ ޽ ְ  Channel Ǵ
 * Session̴.    ȭ  ȴ.
 * <p>
 *
 * @author Jang-Ho Hwang, rath@linuxkorea.co.kr
 * @version $Id: SwitchboardSession.java,v 1.8 2002/03/16 17:44:30 xrath Exp $
 */
public class SwitchboardSession extends AbstractProcessor
{
	private static int timeout = 180000;
	private FileMessageProcessor file = null;

	private String sessionId = null;
	private String cookie = null;
	private HashMap friendMap = new HashMap();
	private MsnFriend lastFriend = null;

	public SwitchboardSession( MSNMessenger msn, ServerInfo info, String sessionId )
	{
		super( msn, info, 1 );

		setServerName( "SS" );
		setAutoOutSend( true );

		this.sessionId = sessionId;
		this.file = new FileMessageProcessor(this);
	}

	/**
	 *   id ڿ · ȯش.
	 */
	public String getSessionId()
	{
		return this.sessionId;
	}

	/**
	 *   id ڿ · Ѵ.
	 */
	public void setSessionId( String sessionId )
	{
		this.sessionId = sessionId;
	}

	public String getCookie()
	{
		return this.cookie;
	}

	public void setCookie( String cookie )
	{
		this.cookie = cookie;
	}

	protected void makeConnection() throws IOException
	{
		super.makeConnection();
		setTimeout( timeout );
	}

	/**
	 *  Channel 󸶰 ƹ ޽  ʾ 
	 *  ϰ   ִµ, ̰ Ǵ Timeout millisecond Ѵ.
	 *
	 * @param  timeout  millisecond SO_TIMEOUT
	 */
	public void setTimeout( int timeout )
	{
		this.timeout = timeout;
		if( socket!=null )
		{
			try
			{
				socket.setSoTimeout( timeout );
			}
			catch( IOException e ) {
				System.err.println( "can't assign SO_TIMEOUT value" );
			}
		}
	}

	/**
	 *  Channel 󸶰 ƹ ޽  ʾ 
	 *  ϰ   ִµ, ̰ Ǵ Timeout millisecond ´.
	 * <p>
	 * default 180000(3) Ǿִ.
	 */
	public int getTimeout()
	{
		return this.timeout;
	}

	public void init() throws IOException
	{
		Callback cb = Callback.getInstance("processRosterInfo", this.getClass());
		cb.setInfinite();

		OutgoingMessage out = new OutgoingMessage("ANS");
		markTransactionId( out );
		out.add( msn.getLoginName() );
		out.add( cookie );
		out.add( sessionId );
		out.setBackProcess( cb );

		sendMessage( out );
	}

	/**
	 * Ư ڸ  ǿ ߰Ѵ.
	 */
	protected void addMsnFriend( MsnFriend friend )
	{
		friendMap.put( friend.getLoginName(), friend );
		this.lastFriend = friend;
	}

	/**
	 *  ֱٿ  ǿ  ģ ´.
	 */
	public MsnFriend getMsnFriend()
	{
		return this.lastFriend;
	}

	/**
	 * ־ loginName  ڰ  ǿ ִ ƴ
	 * ȮѴ.
	 */
	public boolean isInFriend( String loginName )
	{
		return friendMap.containsKey(loginName);
	}

	/**
	 * Ư loginName  ڸ  ǿ Ѵ.
	 */
	protected MsnFriend removeMsnFriend( String loginName )
	{
		return (MsnFriend)friendMap.remove( loginName );
	}

	public Collection getMsnFriends()
	{
		return friendMap.values();
	}

	/**
	 *   ǿ Ǿִ ģ  ´.
	 * ڱ ڽ  ڿ ܵȴ.
	 */
	public int getFriendCount()
	{
		return friendMap.size();
	}

	/**
	 *
	 */
	protected void processMimeMessage( IncomingMessage msg ) throws Exception
	{
		/*
		 * MSG UserHandle FriendlyName Length\r\nMessage
		 *
		 *   ϱ ߿  ٲ㺸  .
		 * @version 1.0.000, 2001/11/09
		 */
		CharArrayWriter caw = new CharArrayWriter(6);
		MimeMessage mime = new MimeMessage(/*msg.get(0), msg.get(1)*/);
		int len = msg.getInt(2);
		int readlen = 0;
		byte[] b = new byte[ len ];
		int buf;
		while( true )
		{
			buf = in.read();
			if( buf==-1 )
				break;
			caw.write( buf );
			byte[] currentBytes = new String(caw.toCharArray()).getBytes("UTF-8");
			System.arraycopy( currentBytes, 0, b, readlen, currentBytes.length );
			readlen += currentBytes.length;

			if( !(readlen < len) )
				break;
			caw.reset();
		}
		String s = new String(b,0,readlen,"UTF-8");
		mime.parse( s );
		if( Debug.printMime )
			System.out.println( s );

		int kind = mime.getKind();
		switch( kind )
		{
		case MimeMessage.KIND_TYPING_USER:
			processTypingUser( msg, mime );
			break;
		case MimeMessage.KIND_MESSAGE:
			processInstantMessage( msg, mime );
			break;
		case MimeMessage.KIND_FILE_TRANSFER:
			file.processMessage( msg, mime );
			break;
		case MimeMessage.KIND_UNKNOWN:
			break;
		}
	}

	protected void processWhoJoined( IncomingMessage msg ) throws Exception
	{
		String loginName = msg.get(0);
		String friendlyName = msg.get(1);

		MsnFriend friend = new MsnFriend(loginName, friendlyName);
		addMsnFriend( friend );
		msn.fireJoinSessionEvent( this, friend );
	}

	/**
	 * Switchboard Serverκ notifyǴ ޽ óѴ.
	 */
	public void processNotifyMessage( IncomingMessage msg ) throws Exception
	{
		String header = msg.getHeader();
		if( header.equals("MSG") )
			processMimeMessage(msg);
		else
		if( header.equals("JOI") )
			processWhoJoined(msg);
		else
		if( header.equals("BYE") )
		{
			String partLoginName = msg.get(0);
			MsnFriend parter = removeMsnFriend( partLoginName );

			if( parter!=null )
				msn.firePartSessionEvent( this, parter );
			if( friendMap.size()==0 )
			{
				isLive = false;
			}
		}
	}

	/**
	 * 尡 ǰ Ʈ ݱ  ȣȴ.
	 */
	public void cleanUp()
	{
		try
		{
			close();
		}
		catch( IOException e ) {}
		friendMap.clear();
		if( sessionId!=null )
			msn.fireSwitchboardSessionEndedEvent( this );
	}

	/**
	 *   Ѵ.
	 */
	public void close() throws IOException
	{
		isLive = false;

		OutgoingMessage out = new OutgoingMessage("OUT");
		sendMessage( out );
		interrupt();
	}

	public void processRosterInfo( IncomingMessage msg ) throws IOException
	{
		String header = msg.getHeader();
		if( header.equals("IRO") )
		{
			String destLoginName = msg.get(2);
			String destFriendlyName = msg.get(3);

			MsnFriend friend = new MsnFriend(destLoginName);
			friend.setFriendlyName( destFriendlyName );

			addMsnFriend( friend );
		}
		else
		if( header.equals("ANS") )
		{
			removeInfiniteTransactionId( msg.getTransactionId() );
			String returnCode = msg.get(0);
			if( returnCode.equals("OK") )
			{
				msn.fireSwitchboardSessionStartedEvent( this );
			}
		}
	}

	/**
	 *  ڽ  νƮ ޽   Ű带
	 * ٴڴٴ ε  <b></b>  ޽̴.
	 */
	protected void processTypingUser( IncomingMessage msg, MimeMessage mime )
		throws IOException
	{
		//  ޼ҵ ̺Ʈ ʿ ؾѴ.
		MsnFriend friend = new MsnFriend( msg.get(0), msg.get(1) );
		msn.fireProgressTypingEvent( this, friend, mime.getProperty("TypingUser") );
	}

	/**
	 * νƮ ޽ Ͽ  ޽ óϴ ޼ҵ̴.
	 */
	protected void processInstantMessage( IncomingMessage msg, MimeMessage mime )
		throws IOException
	{
		MsnFriend friend = new MsnFriend( msg.get(0) );
		friend.setFriendlyName( msg.get(1) );

		msn.fireInstantMessageEvent( this, friend, mime );
	}

	/**
	 *  ǿ ִ   ޽ .
	 */
	public void sendMessage( MimeMessage mime ) throws IOException
	{
		/*
		 * markTransactionId ݵ ٿ߸ Ѵ.
		 */
		OutgoingMessage out = new OutgoingMessage("MSG");
		markTransactionId( out );
		out.add( "N" );

		sendMimeMessage( out, mime );
	}

	/**
	 *  ǿ    ϰڴٴ ޽ .
	 */
	public void sendFileRequest( ToSendFile file, FileTransferMessage mime ) throws IOException
	{
		this.file.registerSend( file );
		sendMessage( mime );
	}

	/**
	 * κ   û <b></b>Ѵ.
	 *
	 * @param  cookie   û ޾Ҵ Ű.
	 * @param  toReceive  ٿε   .
	 */
	public void acceptFileReceive( int cookie, java.io.File toReceive ) throws IOException
	{
		this.file.registerReceive( cookie, toReceive );
		sendMessage( FileTransferMessage.createAcceptMessage(cookie) );
	}

	/**
	 * κ   û <b></b>Ѵ.
	 *
	 * @param  cookie   û ޾Ҵ Ű.
	 */
	public void rejectFileReceive( int cookie ) throws IOException
	{
		sendMessage( FileTransferMessage.createRejectMessage(cookie) );
	}

	/**
	 * Ÿ ̶ ޽ Ѵ.
	 */
	public void sendTypingMessage( MimeMessage mime ) throws IOException
	{
		mime.setKind( MimeMessage.KIND_TYPING_USER );
		sendMessage( mime );
	}

	/**
	 * νϽ ޽ Ѵ.
	 */
	public void sendInstantMessage( MimeMessage mime ) throws IOException
	{
		mime.setKind( MimeMessage.KIND_MESSAGE );
		sendMessage( mime );
	}

	/**
	 *   ģ ʴѴ.
	 */
	public void inviteFriend( String loginName ) throws IOException
	{
		OutgoingMessage out = new OutgoingMessage( "CAL" );
		markTransactionId( out );
		out.add( loginName );

		sendMessage( out );
	}

	public void processError( Throwable e )
	{
		if( !(e instanceof IOException) )
			e.printStackTrace();
	}
};
