/*
 * @(#)FileMessageProcessor.java
 *
 * Copyright (c) 2001-2002, JangHo Hwang
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 	1. Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 * 
 * 	2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 * 
 * 	3. Neither the name of the JangHo Hwang nor the names of its contributors
 * may be used to endorse or promote products derived from this software
 * without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 *    $Id: FileMessageProcessor.java,v 1.2 2002/03/05 13:35:33 xrath Exp $ 
 */
package rath.msnm.ftp;

import java.io.IOException;
import java.io.File;
import java.io.FileNotFoundException;
import java.util.Hashtable;

import rath.msnm.SwitchboardSession;
import rath.msnm.msg.IncomingMessage;
import rath.msnm.msg.MimeMessage;
import rath.msnm.msg.FileTransferMessage;
/**
 *  ۼſ Ǵ  Ʈǵ ϰ  õ Cookie 
 *   ϴ thread µ   óϴ Ŭ̴.
 * <p>
 * Ʒ  Ŭ   ִ  Ѵ.
 * <ul>
 * 
 * </ul>
 * 
 * @author Jang-Ho Hwang, rath@linuxkorea.co.kr
 * @version $Id: FileMessageProcessor.java,v 1.2 2002/03/05 13:35:33 xrath Exp $ 
 */
public class FileMessageProcessor 
{
	private static boolean _auto = true;
	private SwitchboardSession session = null;
	private Hashtable toSendFileMap = new Hashtable();
	private Hashtable toReceiveFileMap = new Hashtable();

	/**
	 +* session Ͼ  File ۿ õ ޽ 
	 * óϰ ϴ ü Ѵ.
	 */
	public FileMessageProcessor( SwitchboardSession session )
	{
		this.session = session;
	}

	/**
	 *  ۼſ õ ޽ óϴ  ޼ҵ鿡 
	 * Entry point Ǵ ޼ҵ̴.
	 *
	 * @param  msg  κ  ޽
	 * @param  mime   ۿ õ   Mime ޽ FileTransferMessage̴. 
	 *               , FileTransferMessage νϽ ƴϹǷ cast   .
	 */
	public void processMessage( IncomingMessage msg, MimeMessage mime ) 
	{			
		// mime.getProperties().list( System.out );
		String command = mime.getProperty("Invitation-Command");
		if( command==null || !mime.hasProperty("Invitation-Cookie") ) 
			return;

		boolean isLastInTransaction = mime.hasProperty("AuthCookie");

		try
		{
			if( command.equals("INVITE") )
			{
				processInvite(mime);
			}
			else
			if( command.equals("ACCEPT") )
			{
				if( !isLastInTransaction )
					processAccept(mime);
				else
					processDownload(mime);
			}
			else
			if( command.equals("CANCEL") )
			{
				processCancel(mime);
			}
		}
		catch( IOException e )
		{
			e.printStackTrace();
		}
		catch( Throwable e )
		{
			System.err.println( "MIME message violation: FileTransfer(" +e+ ")" );
		}
	}

	/**
	 * κ   û óѴ.
	 *  ޼ҵ忡 MIME ޽ ʿ   MsnFileEvent ߼ϴ  Ѵ.
	 * , ${@link #setAutoReceive setAutoReceive} ޼ҵ ڵ ɼ true سҴٸ
	 * ̺Ʈ ߼ ʰ ٷ  ޽ ۽Ѵ.
	 * <p>
	 * ڵ Ÿ尡 Ǿ , ŵǴ  ⺻ current directory 
	 *  ̴.  ̰ ϰ ʹٸ, System property 
	 * <b>msnm.file.download.dir</b> ްϴ 丮 ϸ ȴ.
	 */
	protected void processInvite( MimeMessage mime ) throws IOException
	{
		String cookie = mime.getProperty("Invitation-Cookie");
		int cookieInt = Integer.parseInt(cookie);
		String filename = mime.getProperty("Application-File");
		int filesize = Integer.parseInt(mime.getProperty("Application-FileSize"));

		if( _auto )
		{
			session.acceptFileReceive(cookieInt, 
				new File(System.getProperty("msnm.file.download.dir","."), filename));
			return;
		}
		
		session.msn.fireFilePostedEvent( session, cookieInt, filename, filesize );
	}

	/**
	 *  ڵ 带 Ѵ. 
	 */
	public static void setAutoReceive( boolean auto )
	{
		_auto = auto;
	}

	/**
	 *  ڵ  ȮѴ. default true̴.
	 */
	public static boolean isAutoReceive()
	{
		return _auto;
	}

	/**
	 *  ۽ û ߴٴ   session Ѵ. 
	 *  ޼ҵ α׷Ӱ  ȣ ʿ . 
	 * public   , rath.msnm.SwitchboardSession Ŭ
	 * sendFileRequest޼ҵ忡 registerSend ȣؾߵ ʿ䰡 ֱ ̴.
	 * <p>
	 * Ͽ    / ޽  ڵ ȴ.
	 *   ʴ´ٸ,  Map  ʴ´.
	 *
	 * @param  cookie  ۽ Ʈǿ  Ű
	 */
	public void registerSend( ToSendFile file )
	{
		toSendFileMap.put( file.getCookie(), file );
	}

	/** 
	 *   ߴٴ   session Ѵ.
	 *  ޼ҵ α׷Ӱ  ȣ ʿ .
	 * <p>
	 * Ͽ    ۽ 㰡޽  ڵ ȴ.
	 *   ʴ´ٸ,  Map  ʴ´.
	 */
	public void registerReceive( int cookie, File file )
	{
		toReceiveFileMap.put( String.valueOf(cookie), file );
	}

	/**
	 * ۽ſû ڰ ''Ͽٴ ޽ óѴ.
	 * <p>
	 * ̺Ʈ ߼۽Ų , File ϱ  FileServer thread ϰ
	 * Ѵ.   registerSend ϵ  Map ŵȴ.
	 */
	public void processAccept( MimeMessage mime ) throws FileNotFoundException, IOException
	{
		String cookie = mime.getProperty("Invitation-Cookie");
		ToSendFile toSend = (ToSendFile)toSendFileMap.remove(cookie);
		if( toSend!=null )
		{
			int cookieInt = Integer.parseInt(cookie);
			FileTransferMessage msg = FileTransferMessage.createTransferMessage(cookieInt);

			VolatileTransferServer vts = VolatileTransferServer.getInstance( session.msn, toSend, msg );
			vts.cookie = cookie;

			session.sendMessage( msg );
			session.msn.fireFileSendAcceptedEvent( session, cookieInt );

			vts.start();
		}
	}

	/**
	 * ۽ſû ڰ ''Ͽٴ ޽ óѴ.
	 * <p>
	 * ̺Ʈ ߼۽Ų , registerSend ϵ  Map ŵȴ.
	 */
	public void processCancel( MimeMessage mime )
	{
		String cookie = mime.getProperty("Invitation-Cookie");
		ToSendFile toSend = (ToSendFile)toSendFileMap.remove(cookie);
		if( toSend!=null )
		{
			int cookieInt = Integer.parseInt(cookie);
			session.msn.fireFileSendRejectedEvent( session, cookieInt, mime.getProperty("Cancel-Code") );
		}
	}

	/**
	 * toReceiveMap û cookie ִ Ͽ Ἲ Ѿ߸ Ѵ.
	 * 
	 * ۽ ' ޾ư~' ϴ ޽  ó ־ Ѵ.
	 * Mime ޽   AuthCookie, host, port   ۽ 
	 *  Ͽ  ٿε ϰ, BYE~
	 */
	public void processDownload( MimeMessage mime )
	{
		String cookie = mime.getProperty("Invitation-Cookie");
		File toReceive = (File)toReceiveFileMap.remove(cookie);		
		if( toReceive!=null )
		{
			/*   ̹ ص,    Ѵ.
			   ̰ Applicationʿ ذؾ߸ Ѵ. */
			VolatileDownloader vd = VolatileDownloader.getInstance( session.msn,
				toReceive, mime );
			vd.cookie = cookie;
			vd.start();		
		}
	}
}
