/*
 * @(#)FileTransferMessage.java
 *
 * Copyright (c) 2001-2002, JangHo Hwang
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 	1. Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 *
 * 	2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 *
 * 	3. Neither the name of the JangHo Hwang nor the names of its contributors
 * may be used to endorse or promote products derived from this software
 * without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *    $Id: FileTransferMessage.java,v 1.4 2002/03/15 18:40:09 xrath Exp $
 */
package rath.msnm.msg;

import java.io.File;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.net.InetAddress;
import java.net.UnknownHostException;
import java.net.ServerSocket;
import java.util.Random;

/**
 * ۿ  MIME ޽ ս ϰ, ٷ  ְִ
 * Ŭ̴.
 *
 * @author Jang-Ho Hwang, rath@linuxkorea.co.kr
 * @version $Id: FileTransferMessage.java,v 1.4 2002/03/15 18:40:09 xrath Exp $
 */
public class FileTransferMessage extends MimeMessage
{
	private static final Random random = new Random(System.currentTimeMillis());

	private FileTransferMessage()
	{
		setKind( KIND_FILE_TRANSFER );
	}

	/**
	 *  ۽ û ޽ Ѵ. ۽ ϴ ޽̴.
	 * <p>
	 * Ƿ  Ű ˰ Ѵٸ,  ۸޽ ü
	 * getProperty("Invitation-Cookie")  invokeϿ ˾ƺ  ִ.
	 *
	 * @param  toSend  ϰ ϴ 
	 */
	public synchronized static FileTransferMessage createInviteMessage( File toSend )
	{
		int cookie = random.nextInt( 999998 ) + 1;
		return createInviteMessage( toSend, cookie );
	}

	/**
	 *  ۽ û ޽ Ѵ. ۽ ϴ ޽̴.
	 *
	 * @param  toSend  ϰ ϴ 
	 * @param  cookie  ʴ Ű. 0 ū ڿ߸ Ѵ.
	 */
	public static FileTransferMessage createInviteMessage( File toSend,
		int cookie )
	{
		if( cookie < 1 )
			throw new IllegalArgumentException( "cookie must larger than 0" );
		if( !toSend.exists() )
			throw new IllegalArgumentException( toSend.getName() + " not found" );

		FileTransferMessage msg = new FileTransferMessage();
		msg.setProperty( "Application-Name", " " );
		msg.setProperty( "Application-GUID", "{5D3E02AB-6190-11d3-BBBB-00C04F795683}" );
		msg.setProperty( "Invitation-Command", "INVITE" );
		msg.setProperty( "Invitation-Cookie", String.valueOf(cookie) );
		msg.setProperty( "Application-File", toSend.getName() );
		msg.setProperty( "Application-FileSize", String.valueOf(toSend.length()) );
		return msg;
	}

	/**
	 *   ϴ ޽ Ѵ.  ϴ ޽̴.
	 *
	 * @param  cookie  ۽ڷκ  Ű.
	 */
	public static FileTransferMessage createAcceptMessage( int cookie )
	{
		FileTransferMessage msg = new FileTransferMessage();
		msg.setProperty( "Invitation-Command", "ACCEPT" );
		msg.setProperty( "Invitation-Cookie", String.valueOf(cookie) );
		msg.setProperty( "Launch-Application", "FALSE" );
		msg.setProperty( "Request-Data", "IP-Address:" );
		return msg;
	}

	/**
	 *   źϴ ޽ Ѵ.  ϴ ޽̴.
	 *
	 * @param  cookie  ۽ڷκ  Ű.
	 */
	public static FileTransferMessage createRejectMessage( int cookie )
	{
		FileTransferMessage msg = new FileTransferMessage();
		msg.setProperty( "Invitation-Command", "CANCEL" );
		msg.setProperty( "Invitation-Cookie", String.valueOf(cookie) );
		msg.setProperty( "Cancel-Code", "REJECT" );
		return msg;
	}

	/**
	 * ΰ  /   ۽  
	 * ڽ    ޽̴. <br>
	 * ɶ  Ű ϳ  ϴµ,   AuthCookie 
	 * key ǹǷ, getProperty ޼ҵ带  ݵ Ȯϵ Ѵ.
	 * <p>
	 *  ޽ ϸ ٷ  ϰ ǹǷ bind  Ŀ
	 * ؾ߸ Ѵ.
	 * <p>
	 *  ڽ ip InetAddress.getLocalHost() Ͽ .<br>
	 * port ȣ ⺻ 6891 ϰ ȴ.
	 * <p>
	 *  ش Ʈȣ ̶, 6891~7000 ٸ Ʈȣ Ѵ.
	 *
	 * @param  cookie  ʱ Ͽ Ű.
	 */
	public static FileTransferMessage createTransferMessage( int cookie )
	{
		return createTransferMessage( cookie, 6891 );
	}

	/**
	 * ΰ  /   ۽  
	 * ڽ    ޽̴. <br>
	 * ɶ  Ű ϳ  ϴµ,   AuthCookie 
	 * key ǹǷ, getProperty ޼ҵ带  ݵ Ȯϵ Ѵ.
	 * <p>
	 *  ޽ ϸ ٷ  ϰ ǹǷ bind  Ŀ
	 * ؾ߸ Ѵ.
	 * <p>
	 *  ڽ ip InetAddress.getLocalHost() Ͽ .<br>
	 *  ϴ portȣ ̹ bindȰ̶ 7000 1 ϸ bind
	 * ɶ ݺϰ ȴ.
	 *
	 * @param  cookie  ʱ Ͽ Ű.
	 * @param  port     ۿ  Ʈ ȣ.
	 */
	public static FileTransferMessage createTransferMessage( int cookie, int port )
	{
		String host = null;
		try
		{
			host = InetAddress.getLocalHost().getHostAddress();
		}
		catch( UnknownHostException e )
		{
			throw new Error("Can't get localhost address");
		}

		FileTransferMessage msg = new FileTransferMessage();
		msg.setProperty( "Invitation-Command", "ACCEPT" );
		msg.setProperty( "Invitation-Cookie", String.valueOf(cookie) );
		msg.setProperty( "AuthCookie", createAuthCookie() );
		msg.setProperty( "Launch-Application", "FALSE" );
		msg.setProperty( "Request-Data", "IP-Address:" );
		msg.setProperty( "IP-Address", host );
		msg.setProperty( "Port", String.valueOf(msg.getAvailablePort(port)) );
		return msg;
	}

	/**
	 * ־ port port+100 1 ϸ鼭  Ʈ ˻Ͽ bind
	 * port ȣ ȯѴ.
	 *
	 * @return  bind Ʈȣ.   Ʈȣ ϳ ٸ -1 ȯѴ.
	 */
	protected synchronized int getAvailablePort( int port )
	{
		for(int i=port, limit=port+100; i<limit; i++)
		{
			ServerSocket socket = null;
			try
			{
				socket = new ServerSocket(i);
			}
			catch( IOException e ) {}
			finally
			{
				try	{
					socket.close();
				} catch( Exception e ) {}
			}
			if( socket!=null )
				return i;
		}
		return -1;
	}

	/**
	 * 8ڸ   Ű س.
	 */
	private static String createAuthCookie()
	{
		StringBuffer sb = new StringBuffer();
		for(int i=0; i<8; i++)
			sb.append( (int)(Math.random()*9.0D) + 1 );
		return sb.toString();
	}

	public byte[] getBytes() throws UnsupportedEncodingException
	{
		if( getKind()!=KIND_FILE_TRANSFER )
			return super.getBytes();

		StringBuffer buf = createMimeHeader( STR_FILE_TRANSFER + "; charset=UTF-8" );
		buf.append( "\r\n" );
		fillMimeProperties( buf );
		buf.append( "\r\n" );
		String msg = buf.toString();
		return msg.getBytes("UTF-8");
	}
}
