/*
 * @(#)MimeMessage.java
 *
 * Copyright (c) 2001-2002, JangHo Hwang
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 	1. Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 *
 * 	2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 *
 * 	3. Neither the name of the JangHo Hwang nor the names of its contributors
 * may be used to endorse or promote products derived from this software
 * without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *    $Id: MimeMessage.java,v 1.8 2002/04/01 15:27:17 xrath Exp $
 */
package rath.msnm.msg;

import java.awt.Color;
import java.io.BufferedReader;
import java.io.StringReader;
import java.io.UnsupportedEncodingException;
import java.util.Properties;
import java.util.Enumeration;
/**
 * MIME ޽ ϴ Ŭ̴.
 *
 * @author Jang-Ho Hwang, rath@linuxkorea.co.kr
 * @version $Id: MimeMessage.java,v 1.8 2002/04/01 15:27:17 xrath Exp $
 */
public class MimeMessage
{
	public static final int KIND_PROFILE = 0;
	public static final int KIND_TYPING_USER = 1;
	public static final int KIND_MESSAGE = 2;
	public static final int KIND_FILE_TRANSFER = 3;
	public static final int KIND_UNKNOWN = 10;

	public static final String STR_PROFILE = "text/x-msmsgsprofile";
	public static final String STR_TYPING_USER = "text/x-msmsgscontrol";
	public static final String STR_MESSAGE = "text/plain";
	public static final String STR_FILE_TRANSFER = "text/x-msmsgsinvite";

	private int kind = 0;

	private String message = null;
	private String ef = "";
	private String fn = "";
	private Color fontColor = null;
	private Properties prop = new Properties();

	public MimeMessage()
	{
		this( null );
	}

	/**
	 * ־ ޽  MIME ޽ νϽ Ѵ.
	 */
	public MimeMessage( String message )
	{
		setMessage( message );
		this.fontColor = getRandomColor();
	}

	/**
	 *  MimeMessage type ȯش.
	 *  αν ޴ profile (KIND_PROFILE),
	 * Typing ߿ ۵Ǵ typing (KIND_TYPING_USER),
	 * νƮ ޽ ۵ɶ Ǵ (KIND_MESSAGE) 
	 * Ǿִ.
	 *      ޽   KIND_UNKNOWN
	 * ȯ ̴.
	 */
	public int getKind()
	{
		return this.kind;
	}

	public void setKind( int kind )
	{
		this.kind = kind;
	}

	public String getFontName()
	{
		return this.fn;
	}

	public void setFontName( String fn )
	{
		this.fn = fn;
	}

	public Color getFontColor()
	{
		return this.fontColor;
	}

	public void setFontColor( Color c )
	{
		this.fontColor = c;
	}

	/**
	 * ϰ ϴ ޽ Ѵ.
	 * ټ ,  ޼ҵ KIND_MESSAGE  쿡
	 * ȴ.
	 */
	public void setMessage( String msg )
	{
		this.message = msg;
	}

	/**
	 * KIND_MESSAGE    νƮ ޽ 
	 * µ ȴ.  KIND_UNKNOWN̿ٸ,
	 * raw  ü ڿ ȯ ̴.
	 */
	public String getMessage()
	{
		return this.message;
	}

	/**
	 * ־ line ĽϿ property ϳ ߰Ѵ.
	 *  Ľ  ٸ false  ̰,
	 *    ִٸ, property ߰  
	 * true ȯ ̴.
	 */
	protected boolean addProperty( String line )
	{
		int i0 = line.indexOf( ": " );
		if( i0==-1 )
			return false;
		String key = line.substring( 0, i0 );
		String value = line.substring( i0+2 );

		prop.setProperty( key, value );
		return true;
	}

	/**
	 * Ƽ Ѵ.
	 *
	 * @param  key  Ƽ Ű.
	 * @param  value  Ƽ .
	 */
	public void setProperty( String key, String value )
	{
		this.prop.setProperty( key, value );
	}

	/**
	 *  Ǿִ Ƽ  ִ Properties ü ´.
	 */
	public Properties getProperties()
	{
		return this.prop;
	}

	/**
	 * Ư Mime header  ´.
	 */
	public String getProperty( String key )
	{
		return this.prop.getProperty(key);
	}

	/**
	 * ־ key  mime properties ϴ  θ ´.
	 */
	public boolean hasProperty( String key )
	{
		return this.prop.containsKey(key);
	}

	/**
	 * UTF-8  raw  MIME ޽ ޾ MIME Ŀ 
	 * Ľ/мϿ ͸ ä.
	 *
	 * @param  raw  String MIME ü ޽.
	 */
	public void parse( String raw ) throws Exception
	{
		BufferedReader br = new BufferedReader( new StringReader(raw) );
		addProperty( br.readLine() ); // MIME-Version
		addProperty( br.readLine() ); // Content-Type

		String contentType = prop.getProperty("Content-Type");

		if( contentType.equals(STR_TYPING_USER) )
		{
			this.kind = KIND_TYPING_USER;
			addProperty( br.readLine() );
		}
		else
		if( contentType.startsWith(STR_PROFILE) )
		{
			this.kind = KIND_PROFILE;
			String buf = null;
			while( true )
			{
				buf = br.readLine();
				if( buf==null )
					break;
				if( !addProperty(buf) )
					break;
			}
		}
		else
		if( contentType.startsWith(STR_FILE_TRANSFER) )
		{
			/*
			 * FileTransferMessage ȯ  ֵ Ѵ.
			 */
			this.kind = KIND_FILE_TRANSFER;
			String buf = null;
			while( true )
			{
				buf = br.readLine();
				if( buf==null )
					break;
				if( buf.trim().length()==0 )
					continue;
				if( !addProperty(buf) )
					break;
			}
		}
		else
		if( contentType.startsWith(STR_MESSAGE) )
		{
			this.kind = KIND_MESSAGE;
			addProperty( br.readLine() );
			br.readLine(); // Temporary skip line.

			StringBuffer sb = new StringBuffer();
			String buf = null;
			while( (buf=br.readLine())!=null )
			{
				sb.append( buf );
				/* Ľؼ ϴ  \n̸ ϸ ȴ.*/
				sb.append( "\n" );
			}
			String str = sb.toString();
			this.message = str.substring( 0, str.length()-1 );
		}
		else
		{
			this.kind = KIND_UNKNOWN;
			System.out.println( "UNKNOWN-Content: " + contentType );
			this.message = raw;
			System.out.println( "Raw message: " );
			System.out.println( "-------------------------------------------" );
			System.out.println( raw );
			System.out.println( "-------------------------------------------" );
		}

		br.close();
	}

	private Color getRandomColor()
	{
		int r = (int)(Math.random() * 256.0D);
		int g = (int)(Math.random() * 256.0D);
		int b = (int)(Math.random() * 256.0D);
		
		if( r > 0xa0 ) r = 0xa0;
		if( g > 0xa0 ) g = 0xa0;
		if( b > 0xa0 ) b = 0xa0;
		return new Color( r, g, b );
	}

	private String getStringAsColor( Color c )
	{
		return Integer.toHexString(
			(c.getBlue()<<16) | (c.getGreen()<<8) | (c.getRed()<<0) );
	}

	public String getEffectCode()
	{
		return this.ef;
	}

	public void setEffectCode( String ef )
	{
		this.ef = ef;
	}

	/**
	 * Mime format  UTF-8 base URL encoded byte array
	 *  ش.
	 */
	public byte[] getInstantMessageBytes() throws UnsupportedEncodingException
	{
		StringBuffer sb = new StringBuffer();
		sb.append(
			"MIME-Version: 1.0" +
			"\r\n" +
			"Content-Type: text/plain; charset=UTF-8" +
			"\r\n" +
			"X-MMS-IM-Format: " +
			"FN=" );
		sb.append( MimeUtility.getURLEncodedString(this.fn, "UTF-8") );
		sb.append( "; EF=" );
		sb.append( ef );
		sb.append( "; CO=" );
		sb.append( getStringAsColor(this.fontColor) );
		sb.append(
			"; " +
			"CS=0" +
			"; " +
			"PF=22" +
			"\r\n\r\n" );
		sb.append( this.message );

		return sb.toString().getBytes("UTF-8");
	}

	public byte[] getTypingMessageBytes() throws UnsupportedEncodingException
	{
		StringBuffer sb = createMimeHeader( STR_TYPING_USER );
		sb.append( "TypingUser: " );
		sb.append( message );
		sb.append( "\r\n\r\n" );
		return sb.toString().getBytes("UTF-8");
	}

	/**
	 * Ư Content-Type  Mime header mime message ش.
	 */
	protected StringBuffer createMimeHeader( String header )
	{
		StringBuffer sb = new StringBuffer();
		sb.append( "MIME-Version: 1.0\r\n" +
			"Content-Type: " );
		sb.append( header );
		sb.append( "\r\n" );
		return sb;
	}

	/**
 	 *    Properties buf ߰ϰ,  \r\n ѹ  ٿְ ȴ.
	 */
	protected StringBuffer fillMimeProperties( StringBuffer buf )
	{
		for( Enumeration e = prop.keys(); e.hasMoreElements(); )
		{
			String key = (String)e.nextElement();
			String value = prop.getProperty(key);

			buf.append( key );
			buf.append( ": " );
			buf.append( value );
			buf.append( "\r\n" );
		}
		buf.append( "\r\n" );
		return buf;
	}

	/**
	 *  (Kind)  ϰ byteȯش.
	 */
	public byte[] getBytes() throws UnsupportedEncodingException
	{
		switch(this.kind)
		{
		case KIND_MESSAGE:
			return getInstantMessageBytes();
		case KIND_TYPING_USER:
			return getTypingMessageBytes();
		}
		return null;
	}
};
